package net.lensfrex.socket.client;

import javafx.application.Application;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.stage.Stage;

import javafx.scene.control.*;
import javafx.scene.layout.*;
import net.lensfrex.socket.client.controller.ClientConnection;
import net.lensfrex.socket.common.Common;
import net.lensfrex.socket.common.Constant;
import net.lensfrex.socket.common.DataPack;
import net.lensfrex.socket.common.MessageData;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.util.Random;

public class Client extends Application {
    public void start() {
        launch();
    }

    @Override
    public void start(Stage stage) throws Exception {
        Scene scene = new Scene(new UI().buildUI(), 515, 509);

        // 主题，不想要或者网络出错的话注释掉就好了
        scene.getStylesheets().add(new URL("https://www.lensfrex.net/light.css").toExternalForm());
//        scene.getStylesheets().add(new URL("https://www.lensfrex.net/dark.css").toExternalForm());
//        scene.getStylesheets().add(new URL("http://127.0.0.1/light.css").toExternalForm());

        stage.setTitle("(￣﹃￣)");
        stage.setScene(scene);

        stage.show();
    }
}

class UI {
    private static final String[] DEFAULT_USERNAMES = {"Rex", "Red", "Ash", "Aether"};

    private Pane pane;
    private VBox rootBox;

    private HBox serverControlVBox;
    private TextField serverTextField;

    private TextArea messageTextArea;

    private HBox userControlVBox;
    private TextField usernameTextField;
    private Button loginButton, logoutButton;

    private HBox messageControlVBox;
    private TextField messageTextField;
    private Button sendButton;

    protected Parent buildUI() {
        pane = new Pane();

        rootBox = new VBox();
        rootBox.setPadding(new Insets(10, 10, 10, 10));
        rootBox.setSpacing(20);
        rootBox.setPrefWidth(500);
        rootBox.setPrefHeight(500);

        messageTextArea = new TextArea("暂无消息...");
        messageTextArea.setPrefHeight(700);
        messageTextArea.setEditable(false);

        rootBox.getChildren().addAll(serverControlPane(), userControlPane(), messageTextArea, messageControlPane());
        pane.getChildren().addAll(rootBox);

        return pane;
    }

    private HBox serverControlPane() {
        serverControlVBox = new HBox();
        serverControlVBox.setSpacing(10);
        serverControlVBox.setAlignment(Pos.CENTER_LEFT);

        Label serverLabel = new Label("服务器地址：");

        serverTextField = new TextField("127.0.0.1:2333");
        serverTextField.setPrefWidth(320);

        serverControlVBox.getChildren().addAll(serverLabel, serverTextField);

        return serverControlVBox;
    }


    private HBox userControlPane() {
        userControlVBox = new HBox();
        userControlVBox.setSpacing(10);
        userControlVBox.setAlignment(Pos.CENTER_LEFT);

        Label usernameLabel = new Label("用户名：");

        String username = DEFAULT_USERNAMES[new Random().nextInt(DEFAULT_USERNAMES.length)];
        usernameTextField = new TextField(username);
        usernameTextField.setPrefWidth(200);

        loginButton = new Button("登录");
        loginButton.setOnAction(e -> {
            try {
                this.changeLoginStatus(true);
            } catch (Exception ex) {
                ex.printStackTrace();
                messageTextArea.textProperty().set(messageTextArea.getText() + "请求服务器时发生错误：" + ex);
            }
        });

        logoutButton = new Button("注销");
        logoutButton.setDisable(true);
        logoutButton.setOnAction(e -> {
            try {
                this.changeLoginStatus(false);
            } catch (Exception ex) {
                ex.printStackTrace();
                messageTextArea.textProperty().set(messageTextArea.getText() + "请求服务器时发生错误：" + ex);
            }
        });

        userControlVBox.getChildren().addAll(usernameLabel, usernameTextField, loginButton, logoutButton);

        return userControlVBox;
    }

    private void changeLoginStatus(boolean login) throws IOException {
        byte[] data = usernameTextField.getText().getBytes(StandardCharsets.UTF_8);

        DataPack responseDataPack = this.sendData(data, login ? Constant.OPCODE_LOGIN : Constant.OPCODE_LOGOUT);
        this.displayResponse(responseDataPack);

        this.switchStatus(login);
    }

    private void switchStatus(boolean online) {
        logoutButton.setDisable(!online);
        loginButton.setDisable(online);
        serverTextField.setDisable(online);
        usernameTextField.setDisable(online);
        messageControlVBox.setDisable(!online);
    }

    private HBox messageControlPane() {
        messageControlVBox = new HBox();
        messageControlVBox.setSpacing(10);
        messageControlVBox.setAlignment(Pos.CENTER_LEFT);
        messageControlVBox.setDisable(true);

        Label usernameLabel = new Label("输入消息：");

        messageTextField = new TextField();
        messageTextField.setPrefWidth(320);

        sendButton = new Button("发送");
        sendButton.setOnAction(e -> {
            try {
                MessageData messageData = new MessageData(usernameTextField.getText(), messageTextField.getText());
                DataPack responseDataPack = this.sendData(messageData.toBytes(), Constant.OPCODE_SEND_MESSAGE);

                messageTextField.textProperty().set("");
                this.displayResponse(responseDataPack);
            } catch (Exception ex) {
                ex.printStackTrace();
                messageTextArea.textProperty().set(messageTextArea.getText() + "请求服务器时发生错误：" + ex);
            }
        });

        messageControlVBox.getChildren().addAll(usernameLabel, messageTextField, sendButton);

        return messageControlVBox;
    }

    private ClientConnection createConnection() throws IOException {
        String[] address = serverTextField.getText().split(":");
        int port = Integer.parseInt(address[1]);

        return new ClientConnection(address[0], port);
    }

    private DataPack sendData(byte[] data, byte[] opcode) throws IOException {
        ClientConnection connection = this.createConnection();

        Common.write(data, opcode, connection.getOutputStream());
        connection.finishOutput();

        InputStream inputStream = connection.getInputStream();
        byte[] header = new byte[4];
        inputStream.read(header);

        DataPack dataPack = Common.readDataPack(inputStream);
        connection.finishInput();

        connection.disconnect();
        return dataPack;
    }

    private void displayResponse(DataPack dataPack) {
        MessageData messageData = MessageData.parse(dataPack.getData());

        messageTextArea.textProperty().set(messageTextArea.getText() +
                String.format("\n服务器[%s]响应：%s", messageData.getIdText(), messageData.getMessageText()));
    }
}