package net.lensfrex.socket.common;

import java.io.ByteArrayOutputStream;
import java.io.Serializable;
import java.util.Arrays;

public class DataPack implements Serializable {
    private final byte[] opcode;
    private final byte[] data;

    public DataPack(byte[] opcode, byte[] data) {
        this.opcode = opcode;
        this.data = data;
    }

    public static DataPack parse(byte[] bytes) {
        int offset = 0;
        byte[] opcode = Arrays.copyOfRange(bytes, offset, offset + Constant.OPCODE_LENGTH);

        offset += Constant.OPCODE_LENGTH;
        int dataLength = Util.bytesToInt32(Arrays.copyOfRange(bytes, offset, offset + Constant.LENGTH_BYTE_SIZE));

        offset += Constant.LENGTH_BYTE_SIZE;
        byte[] data = Arrays.copyOfRange(bytes, offset, offset + dataLength);

        return new DataPack(opcode, data);
    }

    public byte[] toBytes() {
        try {
            int totalLength = Constant.OPCODE_LENGTH + Constant.LENGTH_BYTE_SIZE + data.length;
            ByteArrayOutputStream stream = new ByteArrayOutputStream(totalLength);

            stream.write(opcode);
            stream.write(Util.int32ToBytes(data.length));
            stream.write(data);

            return stream.toByteArray();
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public byte[] getOpcode() {
        return opcode;
    }

    public byte[] getData() {
        return data;
    }

    public int getTotalLength() {
        return Constant.OPCODE_LENGTH + Constant.LENGTH_BYTE_SIZE + data.length;
    }

    public int getDataLength() {
        return data.length;
    }
}
