package net.lensfrex.socket.common;

import java.io.ByteArrayOutputStream;
import java.io.Serializable;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;

public class MessageData implements Serializable {
    private final byte[] id;
    private final byte[] messageData;

    public MessageData(byte[] id, byte[] messageData) {
        this.id = id;
        this.messageData = messageData;
    }

    public MessageData(String id, String messageData) {
        this.id = id.getBytes(StandardCharsets.UTF_8);
        this.messageData = messageData.getBytes(StandardCharsets.UTF_8);
    }

    public static MessageData parse(byte[] data) {
        int offset = 0;
        int idLength = Util.bytesToInt32(Arrays.copyOfRange(data, offset, offset + Constant.LENGTH_BYTE_SIZE));

        offset += Constant.LENGTH_BYTE_SIZE;
        byte[] id = Arrays.copyOfRange(data, offset, offset + idLength);

        offset += idLength;
        int messageDataLength = Util.bytesToInt32(Arrays.copyOfRange(data, offset, offset + Constant.LENGTH_BYTE_SIZE));

        offset += Constant.LENGTH_BYTE_SIZE;
        byte[] messageData = Arrays.copyOfRange(data, offset, offset + messageDataLength);

        return new MessageData(id, messageData);
    }

    public byte[] toBytes() {
        try {
            int byteSize = Constant.LENGTH_BYTE_SIZE * 2 + id.length + messageData.length;
            ByteArrayOutputStream stream = new ByteArrayOutputStream(byteSize);

            stream.write(Util.int32ToBytes(id.length));
            stream.write(id);
            stream.write(Util.int32ToBytes(messageData.length));
            stream.write(messageData);

            return stream.toByteArray();
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public byte[] getId() {
        return id;
    }

    public String getIdText() {
        return new String(id);
    }

    public byte[] getMessageData() {
        return messageData;
    }

    public String getMessageText() {
        return new String(messageData);
    }

    @Override
    public String toString() {
        return "MessageData{" +
                "idLength=" + id.length +
                ", id='" + new String(id) +
                "', messageLength=" + messageData.length +
                ", messageData='" + new String(messageData) +
                ";}";
    }
}