package net.lensfrex.socket.server;

import net.lensfrex.socket.common.*;

import java.io.*;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Arrays;
import java.util.HashSet;

public class Server {
    private static boolean stop = false;

    public void start() {
        Server.stop = false;

        try (ServerSocket serverSocket = new ServerSocket(2333)) {
            while (!stop) {
                // Start a new thread to service more than one client
                new ServerThread(serverSocket.accept()).start();
            }
        } catch (Exception e) {
            System.err.println("Some exception happened: " + e);
        }
    }

    public void stop() {
        Server.stop = true;
    }
}

class ServerThread extends Thread {
    private static final String SERVER_ID = "MASTER-SERVER";
    
    private static final HashSet<User> onlineUsers = new HashSet<>();

    private final Socket socket;

    public ServerThread(Socket socket) {
        this.socket = socket;
    }

    @Override
    public void run() {
        System.out.println("New thread started.");

        try (InputStream inputStream = socket.getInputStream(); OutputStream outputStream = socket.getOutputStream()) {
            byte[] clientMagicHeader = new byte[Constant.MAGIC_HEADER.length];
            if (inputStream.read(clientMagicHeader) == -1) {
                return;
            }

            this.executeAction(inputStream, outputStream);
            socket.shutdownOutput();
            socket.shutdownInput();
            socket.close();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private void executeAction(InputStream inputStream, OutputStream outputStream) throws IOException {
        DataPack dataPack = Common.readDataPack(inputStream);
        byte[] opcode = dataPack.getOpcode();

        if (Arrays.equals(Constant.OPCODE_LOGIN, opcode)) {
            this.login(new String(dataPack.getData()));
            Common.writeMessage(SERVER_ID, Constant.MESSAGE_OK, outputStream);

        } else if (Arrays.equals(Constant.OPCODE_LOGOUT, opcode)) {
            this.logout(new String(dataPack.getData()));
            Common.writeMessage(SERVER_ID, Constant.MESSAGE_OK, outputStream);

        } else if (Arrays.equals(Constant.OPCODE_REQUEST_MESSAGE, opcode)) {
            this.sendRandomMessage(outputStream);

        } else if (Arrays.equals(Constant.OPCODE_SEND_MESSAGE, opcode)) {
            String receivedMessage = this.receiveMessage(dataPack);
            String responseMessage = String.format("Receive message '%s'",  receivedMessage);
            Common.writeMessage(SERVER_ID, responseMessage, outputStream);
        }
    }

    private void login(String id) {
        onlineUsers.add(new User(id));

        System.out.printf("User '%s' login.%n", id);
    }

    private void logout(String id) {
        onlineUsers.remove(new User(id));

        System.out.printf("User '%s' logout.%n", id);
    }

    private void sendRandomMessage(OutputStream outputStream) throws IOException {
        this.sendMessage(Util.randomString(64), outputStream);
    }

    private void sendMessage(String messageText, OutputStream outputStream) throws IOException {
        Common.writeMessage(SERVER_ID, messageText, outputStream);
    }

    private String receiveMessage(DataPack dataPack) {
        byte[] data = dataPack.getData();
        MessageData messageData = MessageData.parse(data);

        User user = new User(messageData.getIdText());

        if (!onlineUsers.contains(user)) {
            String message = "User " + user.getId() + " is not online, but send a message.";
            System.err.println(message);

            return message;
        }

        System.out.printf("Receive message '%s' from: '%s'%n", messageData.getMessageText(), user.getId());
        System.out.println("----------------");

        return messageData.getMessageText();
    }
}